<?php

namespace Drupal\drupaorg_migrate\EventSubscriber;

use Drupal\Core\Cache\MemoryCache\MemoryCacheInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigratePostRowSaveEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Migration event handler for migrations.
 */
class MemoryControlEventHandler implements EventSubscriberInterface {

  /**
   * Keeps track of how many iterations will run until memory cache is flushed.
   */
  protected int $iterationCountUntilCacheFlush = 50;

  public function __construct(
    protected ConfigFactoryInterface $config_factory,
    protected MemoryCacheInterface $memory_cache,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      // @todo Only do if needed.
      // MigrateEvents::POST_ROW_SAVE => ['reduceMemoryUsage']
    ];
  }

  /**
   * Clears memory caches after a limit of items imported to reduce leakage.
   *
   * @param \Drupal\migrate\Event\MigratePostRowSaveEvent $event
   *   The migrate event object.
   */
  public function reduceMemoryUsage(MigratePostRowSaveEvent $event): void {
    $this->iterationCountUntilCacheFlush--;
    if ($this->iterationCountUntilCacheFlush <= 0) {
      $this->memoryCache->deleteAll();
      $this->configFactory->clearStaticCache();
      $this->iterationCountUntilCacheFlush = 50;
    }
  }

}
